@@ -88,21 +88,37 @@ version(Message) :-
                 *         INITIALISATION        *
                 *********************************/
 
-%       note: loaded_init_file/2 is used by prolog_load_context/2 to
-%       confirm we are loading a script.
+%!  load_init_file is det.
+%
+%   Load the user customization file. This can  be done using ``swipl -f
+%   file`` or simply using ``swipl``. In the   first  case we search the
+%   file both directly and over  the   alias  `user_app_config`.  In the
+%   latter case we only use the alias.
+
+load_init_file :-
+    '$cmd_option_val'(init_file, OsFile),
+    !,
+    prolog_to_os_filename(File, OsFile),
+    load_init_file(File, explicit).
+load_init_file :-
+    load_init_file('init.pl', implicit).
+
+%!  loaded_init_file(?Base, ?AbsFile)
+%
+%   Used by prolog_load_context/2 to confirm we are loading a script.
 
 :- dynamic
     loaded_init_file/2.             % already loaded init files
 
-'$load_init_file'(none) :- !.
-'$load_init_file'(Base) :-
+load_init_file(none, _) :- !.
+load_init_file(Base, _) :-
     loaded_init_file(Base, _),
     !.
-'$load_init_file'(InitFile) :-
+load_init_file(InitFile, explicit) :-
     exists_file(InitFile),
     !,
     ensure_loaded(user:InitFile).
-'$load_init_file'(Base) :-
+load_init_file(Base, _) :-
     absolute_file_name(user_app_config(Base), InitFile,
                        [ access(read),
                          file_errors(fail)
@@ -111,7 +127,7 @@ version(Message) :-
     load_files(user:InitFile,
                [ scope_settings(false)
                ]).
-'$load_init_file'('init.pl') :-
+load_init_file('init.pl', implicit) :-
     (   current_prolog_flag(windows, true),
         absolute_file_name(user_profile('swipl.ini'), InitFile,
                            [ access(read),
@@ -122,7 +138,7 @@ version(Message) :-
     ),
     !,
     print_message(warning, backcomp(init_file_moved(InitFile))).
-'$load_init_file'(_).
+load_init_file(_, _).
 
 '$load_system_init_file' :-
     loaded_init_file(system, _),
@@ -501,9 +517,7 @@ initialise_prolog :-
     init_debug_flags,
     start_pldoc,
     opt_attach_packs,
-    '$cmd_option_val'(init_file, OsFile),
-    prolog_to_os_filename(File, OsFile),
-    '$load_init_file'(File),
+    load_init_file,
     catch(setup_colors, E, print_message(warning, E)),
     '$load_script_file',
     associated_files(Files),
@@ -1088,30 +1102,34 @@ subst_chars([H|T]) -->
 '$execute_goal2'(Goal, Bindings, true) :-
     restore_debug,
     '$current_typein_module'(TypeIn),
-    residue_vars(TypeIn:Goal, Vars, TypeIn:Delays),
+    residue_vars(TypeIn:Goal, Vars, TypeIn:Delays, Chp),
     deterministic(Det),
     (   save_debug
     ;   restore_debug, fail
     ),
     flush_output(user_output),
+    (   Det == true
+    ->  DetOrChp = true
+    ;   DetOrChp = Chp
+    ),
     call_expand_answer(Bindings, NewBindings),
-    (    \+ \+ write_bindings(NewBindings, Vars, Delays, Det)
+    (    \+ \+ write_bindings(NewBindings, Vars, Delays, DetOrChp)
     ->   !
     ).
 '$execute_goal2'(_, _, false) :-
     save_debug,
     print_message(query, query(no)).
 
-residue_vars(Goal, Vars, Delays) :-
+residue_vars(Goal, Vars, Delays, Chp) :-
     current_prolog_flag(toplevel_residue_vars, true),
     !,
-    '$wfs_call'(call_residue_vars(stop_backtrace(Goal), Vars), Delays).
-residue_vars(Goal, [], Delays) :-
-    '$wfs_call'(stop_backtrace(Goal), Delays).
+    '$wfs_call'(call_residue_vars(stop_backtrace(Goal, Chp), Vars), Delays).
+residue_vars(Goal, [], Delays, Chp) :-
+    '$wfs_call'(stop_backtrace(Goal, Chp), Delays).
 
-stop_backtrace(Goal) :-
+stop_backtrace(Goal, Chp) :-
     toplevel_call(Goal),
-    no_lco.
+    prolog_current_choice(Chp).
 
 toplevel_call(Goal) :-
     call(Goal),
@@ -1119,7 +1137,7 @@ toplevel_call(Goal) :-
 
 no_lco.
 
-%!  write_bindings(+Bindings, +ResidueVars, +Delays +Deterministic)
+%!  write_bindings(+Bindings, +ResidueVars, +Delays, +DetOrChp)
 %!	is semidet.
 %
 %   Write   bindings   resulting   from   a     query.    The   flag
@@ -1133,12 +1151,12 @@ no_lco.
 %        the prolog flag `toplevel_residue_vars` is set to
 %        `project`.
 
-write_bindings(Bindings, ResidueVars, Delays, Det) :-
+write_bindings(Bindings, ResidueVars, Delays, DetOrChp) :-
     '$current_typein_module'(TypeIn),
     translate_bindings(Bindings, Bindings1, ResidueVars, TypeIn:Residuals),
     omit_qualifier(Delays, TypeIn, Delays1),
     name_vars(Bindings1, Residuals, Delays1),
-    write_bindings2(Bindings1, Residuals, Delays1, Det).
+    write_bindings2(Bindings1, Residuals, Delays1, DetOrChp).
 
 write_bindings2([], Residuals, Delays, _) :-
     current_prolog_flag(prompt_alternatives_on, groundness),
@@ -1148,10 +1166,10 @@ write_bindings2(Bindings, Residuals, Delays, true) :-
     current_prolog_flag(prompt_alternatives_on, determinism),
     !,
     print_message(query, query(yes(Bindings, Delays, Residuals))).
-write_bindings2(Bindings, Residuals, Delays, _Det) :-
+write_bindings2(Bindings, Residuals, Delays, Chp) :-
     repeat,
         print_message(query, query(more(Bindings, Delays, Residuals))),
-        get_respons(Action),
+        get_respons(Action, Chp),
     (   Action == redo
     ->  !, fail
     ;   Action == show_again
@@ -1220,6 +1238,7 @@ residual_goals(NonTerminal) :-
 
 system:term_expansion((:- residual_goals(NonTerminal)),
                       '$toplevel':residual_goal_collector(M2:Head)) :-
+    \+ current_prolog_flag(xref, true),
     prolog_load_context(module, M),
     strip_module(M:NonTerminal, M2, Head),
     '$must_be'(callable, Head).
@@ -1575,52 +1594,55 @@ hide_names([Name|T0], Skel, Subst, [Name|T]) :-
 self_bounded(binding([Name], Value, [])) :-
     Value == '$VAR'(Name).
 
-%!  get_respons(-Action)
+%!  get_respons(-Action, +Chp)
 %
 %   Read the continuation entered by the user.
 
-get_respons(Action) :-
+get_respons(Action, Chp) :-
     repeat,
         flush_output(user_output),
         get_single_char(Char),
-        answer_respons(Char, Action),
+        answer_respons(Char, Chp, Action),
         (   Action == again
         ->  print_message(query, query(action)),
             fail
         ;   !
         ).
 
-answer_respons(Char, again) :-
+answer_respons(Char, _, again) :-
     '$in_reply'(Char, '?h'),
     !,
     print_message(help, query(help)).
-answer_respons(Char, redo) :-
+answer_respons(Char, _, redo) :-
     '$in_reply'(Char, ';nrNR \t'),
     !,
     print_message(query, if_tty([ansi(bold, ';', [])])).
-answer_respons(Char, redo) :-
+answer_respons(Char, _, redo) :-
     '$in_reply'(Char, 'tT'),
     !,
     trace,
     save_debug,
     print_message(query, if_tty([ansi(bold, '; [trace]', [])])).
-answer_respons(Char, continue) :-
+answer_respons(Char, _, continue) :-
     '$in_reply'(Char, 'ca\n\ryY.'),
     !,
     print_message(query, if_tty([ansi(bold, '.', [])])).
-answer_respons(0'b, show_again) :-
+answer_respons(0'b, _, show_again) :-
     !,
     break.
-answer_respons(Char, show_again) :-
+answer_respons(0'*, Chp, show_again) :-
+    !,
+    print_last_chpoint(Chp).
+answer_respons(Char, _, show_again) :-
     print_predicate(Char, Pred, Options),
     !,
     print_message(query, if_tty(['~w'-[Pred]])),
     set_prolog_flag(answer_write_options, Options).
-answer_respons(-1, show_again) :-
+answer_respons(-1, _, show_again) :-
     !,
     print_message(query, halt('EOF')),
     halt(0).
-answer_respons(Char, again) :-
+answer_respons(Char, _, again) :-
     print_message(query, no_action(Char)).
 
 print_predicate(0'w, [write], [ quoted(true),
@@ -1633,6 +1655,18 @@ print_predicate(0'p, [print], [ quoted(true),
                               ]).
 
 
+print_last_chpoint(Chp) :-
+    current_predicate(print_last_choice_point/0),
+    !,
+    print_last_chpoint_(Chp).
+print_last_chpoint(Chp) :-
+    use_module(library(prolog_stack), [print_last_choicepoint/2]),
+    print_last_chpoint_(Chp).
+
+print_last_chpoint_(Chp) :-
+    print_last_choicepoint(Chp, [message_level(information)]).
+
+
                  /*******************************
                  *          EXPANSION           *
                  *******************************/
