@@ -53,7 +53,7 @@
 /** <module> Check installation issues and features
 
 This library performs checks on  the   installed  system to verify which
-optional components are available and  whether   all  libaries that load
+optional components are available and  whether  all  libraries that load
 shared objects/DLLs can be loaded.
 */
 
@@ -81,6 +81,10 @@ shared objects/DLLs can be loaded.
 %     features.
 
 % Feature tests
+component(tcmalloc,
+          _{ test:test_tcmalloc,
+             url:'tcmalloc.html'
+           }).
 component(gmp,
           _{ test:current_prolog_flag(bounded, false),
              url:'gmp.html'
@@ -239,7 +243,7 @@ current_os(linux)   :- current_prolog_flag(arch, Arch), sub_atom(Arch, _, _, _,
 
 %!  test_component(+Properties) is semidet.
 %
-%   Run additional tests to see whether the componnent really works.
+%   Run additional tests to see whether the component really works.
 
 test_component(Dict) :-
     Test = Dict.get(test),
@@ -312,6 +316,17 @@ error_kind(error).
                  *         SPECIAL TESTS        *
                  *******************************/
 
+%!  test_tcmalloc
+
+:- if(current_predicate(malloc_property/1)).
+test_tcmalloc :-
+    malloc_property('generic.current_allocated_bytes'(Bytes)),
+    Bytes > 1 000 000.
+:- else.
+test_tcmalloc :-
+    fail.
+:- endif.
+
 %!  archive_features
 %
 %   Report features supported by library(archive).
