@@ -298,6 +298,7 @@ pack_info_term(version(version)).
 pack_info_term(author(atom, email_or_url_or_empty)).     % Persons
 pack_info_term(maintainer(atom, email_or_url)).
 pack_info_term(packager(atom, email_or_url)).
+pack_info_term(pack_version(nonneg)).           % Package convention version
 pack_info_term(home(atom)).                     % Home page
 pack_info_term(download(atom)).                 % Source
 pack_info_term(provides(atom)).                 % Dependencies
@@ -637,6 +638,8 @@ url_menu_item(URL, URL=install_from(URL)).
 %     `if_absent` (default, do nothing if the directory with foreign
 %     resources exists), `make` (run `make`) or `true` (run `make
 %     distclean` followed by the default configure and build steps).
+%     * test(Boolean)
+%     If `true` (default), run the pack tests.
 %     * git(+Boolean)
 %     If `true` (default `false` unless `URL` ends with =.git=),
 %     assume the URL is a GIT repository.
@@ -1087,9 +1090,21 @@ pack_rebuild(Pack) :-
     current_pack(Pack, PackDir),
     !,
     post_install_foreign(Pack, PackDir, [rebuild(true)]).
+pack_rebuild(Pack) :-
+    unattached_pacth(Pack, PackDir),
+    !,
+    post_install_foreign(Pack, PackDir, [rebuild(true)]).
 pack_rebuild(Pack) :-
     existence_error(pack, Pack).
 
+unattached_pacth(Pack, BaseDir) :-
+    directory_file_path(Pack, 'pack.pl', PackFile),
+    absolute_file_name(pack(PackFile), PackPath,
+                       [ access(read),
+                         file_errors(fail)
+                       ]),
+    file_directory_name(PackPath, BaseDir).
+
 %!  pack_rebuild is det.
 %
 %   Rebuild foreign components of all packages.
@@ -1108,16 +1123,24 @@ pack_rebuild :-
 post_install_foreign(Pack, PackDir, Options) :-
     is_foreign_pack(PackDir, _),
     !,
+    (   pack_info_term(PackDir, pack_version(Version))
+    ->  true
+    ;   Version = 1
+    ),
     option(rebuild(Rebuild), Options, if_absent),
     (   Rebuild == if_absent,
         foreign_present(PackDir)
     ->  print_message(informational, pack(kept_foreign(Pack)))
     ;   BuildSteps0 = [[dependencies], [configure], build, [test], install],
         (   Rebuild == true
-        ->  BuildSteps = [distclean|BuildSteps0]
-        ;   BuildSteps = BuildSteps0
+        ->  BuildSteps1 = [distclean|BuildSteps0]
+        ;   BuildSteps1 = BuildSteps0
         ),
-        build_steps(BuildSteps, PackDir, Options)
+        (   option(test(false), Options)
+        ->  delete(BuildSteps1, [test], BuildSteps)
+        ;   BuildSteps = BuildSteps1
+        ),
+        build_steps(BuildSteps, PackDir, [pack_version(Version)|Options])
     ).
 post_install_foreign(_, _, _).
 
@@ -1250,13 +1273,8 @@ pack_remove_forced(Pack) :-
     print_message(informational, pack(remove(BaseDir))),
     delete_directory_and_contents(BaseDir).
 pack_remove_forced(Pack) :-
-    directory_file_path(Pack, 'pack.pl', PackFile),
-    absolute_file_name(pack(PackFile), PackPath,
-                       [ access(read),
-                         file_errors(fail)
-                       ]),
+    unattached_pacth(Pack, BaseDir),
     !,
-    file_directory_name(PackPath, BaseDir),
     delete_directory_and_contents(BaseDir).
 pack_remove_forced(Pack) :-
     print_message(informational, error(existence_error(pack, Pack),_)).
