@@ -68,7 +68,7 @@ extern "C" {
 /* PLVERSION_TAG: a string, normally "", but for example "rc1" */
 
 #ifndef PLVERSION
-#define PLVERSION 80503
+#define PLVERSION 80505
 #endif
 #ifndef PLVERSION_TAG
 #define PLVERSION_TAG ""
@@ -198,7 +198,7 @@ typedef _PLS(record) *	record_t;	/* Prolog recorded term */
 #define PL_HAVE_TERM_T
 typedef uintptr_t	term_t;		/* opaque term handle */
 #endif
-typedef uintptr_t	qid_t;		/* opaque query handle */
+typedef _PLS(queryRef) *qid_t;		/* opaque query handle */
 typedef uintptr_t	PL_fid_t;	/* opaque foreign context handle */
 typedef _PLS(foreign_context) *control_t; /* non-deterministic control arg */
 typedef _PLS(PL_local_data) *PL_engine_t; /* opaque engine handle */
@@ -321,12 +321,15 @@ typedef union
 #define PL_CUTTED		(1)	/* deprecated */
 #define PL_PRUNED		(1)
 #define PL_REDO			(2)
+#define PL_RESUME		(3)
 
 #define PL_retry(n)		return _PL_retry(n)
 #define PL_retry_address(a)	return _PL_retry_address(a)
+#define PL_yield_address(a)	return _PL_yield_address(a)
 
 PL_EXPORT(foreign_t)	_PL_retry(intptr_t);
 PL_EXPORT(foreign_t)	_PL_retry_address(void *);
+PL_EXPORT(foreign_t)	_PL_yield_address(void *);
 PL_EXPORT(int)		PL_foreign_control(control_t);
 PL_EXPORT(intptr_t)	PL_foreign_context(control_t);
 PL_EXPORT(void *)	PL_foreign_context_address(control_t);
@@ -413,7 +416,7 @@ PL_EXPORT(const atom_t) *_PL_atoms(void); /* base of reserved (meta-)atoms */
 #define PL_S_FALSE		0	/* Query failed */
 #define PL_S_TRUE		1	/* Query succeeded with choicepoint */
 #define PL_S_LAST		2	/* Query succeeded without CP */
-
+#define PL_S_YIELD	      255	/* Foreign yield */
 
 			/* Foreign context frames */
 PL_EXPORT(fid_t)	PL_open_foreign_frame(void);
@@ -436,6 +439,8 @@ PL_EXPORT(int)		PL_next_solution(qid_t qid) WUNUSED;
 PL_EXPORT(int)		PL_close_query(qid_t qid);
 PL_EXPORT(int)		PL_cut_query(qid_t qid);
 PL_EXPORT(qid_t)	PL_current_query(void);
+PL_EXPORT(PL_engine_t)	PL_query_engine(qid_t qid);
+PL_EXPORT(int)		PL_can_yield(void);
 
 			/* Simplified (but less flexible) call-back */
 PL_EXPORT(int)		PL_call(term_t t, module_t m);
@@ -950,34 +955,35 @@ PL_EXPORT(IOSTREAM *)*_PL_streams(void);	/* base of streams */
 #define Scurrent_output (_PL_streams()[4])
 #endif
 
-#define PL_WRT_QUOTED		0x01	/* quote atoms */
-#define PL_WRT_IGNOREOPS	0x02	/* ignore list/operators */
-#define PL_WRT_NUMBERVARS	0x04	/* print $VAR(N) as a variable */
-#define PL_WRT_PORTRAY		0x08	/* call portray */
-#define PL_WRT_CHARESCAPES	0x10	/* Output ISO escape sequences */
-#define PL_WRT_BACKQUOTED_STRING 0x20	/* Write strings as `...` */
-					/* Write attributed variables */
-#define PL_WRT_ATTVAR_IGNORE	0x040	/* Default: just write the var */
-#define PL_WRT_ATTVAR_DOTS	0x080	/* Write as Var{...} */
-#define PL_WRT_ATTVAR_WRITE	0x100	/* Write as Var{Attributes} */
-#define PL_WRT_ATTVAR_PORTRAY	0x200	/* Use Module:portray_attrs/2 */
+#define PL_WRT_QUOTED		       0x01 /* quote atoms */
+#define PL_WRT_IGNOREOPS	       0x02 /* ignore list/operators */
+#define PL_WRT_NUMBERVARS	       0x04 /* print $VAR(N) as a variable */
+#define PL_WRT_PORTRAY		       0x08 /* call portray */
+#define PL_WRT_CHARESCAPES	       0x10 /* Output ISO escape sequences */
+#define PL_WRT_BACKQUOTED_STRING       0x20 /* Write strings as `...` */
+					    /* Write attributed variables */
+#define PL_WRT_ATTVAR_IGNORE	      0x040 /* Default: just write the var */
+#define PL_WRT_ATTVAR_DOTS	      0x080 /* Write as Var{...} */
+#define PL_WRT_ATTVAR_WRITE	      0x100 /* Write as Var{Attributes} */
+#define PL_WRT_ATTVAR_PORTRAY	      0x200 /* Use Module:portray_attrs/2 */
 #define PL_WRT_ATTVAR_MASK \
 	(PL_WRT_ATTVAR_IGNORE | \
 	 PL_WRT_ATTVAR_DOTS | \
 	 PL_WRT_ATTVAR_WRITE | \
 	 PL_WRT_ATTVAR_PORTRAY)
-#define PL_WRT_BLOB_PORTRAY	0x400	/* Use portray to emit non-text blobs */
-#define PL_WRT_NO_CYCLES	0x800	/* Never emit @(Template,Subst) */
-#define PL_WRT_NEWLINE	       0x2000	/* Add a newline */
-#define PL_WRT_VARNAMES	       0x4000	/* Internal: variable_names(List)  */
-#define PL_WRT_BACKQUOTE_IS_SYMBOL 0x8000 /* ` is a symbol char */
-#define PL_WRT_DOTLISTS	       0x10000	/* Write lists as .(A,B) */
-#define PL_WRT_BRACETERMS      0x20000	/* Write {A} as {}(A) */
-#define PL_WRT_NODICT	       0x40000	/* Do not write dicts in pretty syntax */
-#define PL_WRT_NODOTINATOM     0x80000	/* never write a.b unquoted */
-#define PL_WRT_NO_LISTS	       0x100000	/* Do not write lists as [...] */
-#define PL_WRT_RAT_NATURAL     0x200000	/* Write rationals as 1/3 */
-#define PL_WRT_CHARESCAPES_UNICODE 0x400000 /* Output escape sequences as \uXXXX */
+#define PL_WRT_BLOB_PORTRAY	      0x400 /* Use portray for non-text blobs */
+#define PL_WRT_NO_CYCLES	      0x800 /* Never emit @(Template,Subst) */
+#define PL_WRT_NEWLINE	             0x2000 /* Add a newline */
+#define PL_WRT_VARNAMES	             0x4000 /* Internal: variable_names(List) */
+#define PL_WRT_BACKQUOTE_IS_SYMBOL   0x8000 /* ` is a symbol char */
+#define PL_WRT_DOTLISTS	            0x10000 /* Write lists as .(A,B) */
+#define PL_WRT_BRACETERMS           0x20000 /* Write {A} as {}(A) */
+#define PL_WRT_NODICT	            0x40000 /* Do not write dicts pretty */
+#define PL_WRT_NODOTINATOM          0x80000 /* never write a.b unquoted */
+#define PL_WRT_NO_LISTS	           0x100000 /* Do not write lists as [...] */
+#define PL_WRT_RAT_NATURAL         0x200000 /* Write rationals as 1/3 */
+#define PL_WRT_CHARESCAPES_UNICODE 0x400000 /* Use \uXXXX escapes */
+#define PL_WRT_QUOTE_NON_ASCII	   0x800000 /* Quote atoms containing non-ascii */
 
 PL_EXPORT(int)	PL_write_term(IOSTREAM *s,
 			     term_t term,
