@@ -656,7 +656,17 @@ discardForeignFrame(DECL_LD LocalFrame fr)
   DEBUG(5, Sdprintf("\tCut %s, context = %p\n",
 		    predicateName(def), fr->clause));
 
-  context.context = (word)fr->clause;
+  switch((word)fr->clause & FRG_REDO_MASK)
+  { case REDO_INT:
+      context.context = (word)fr->clause >> FRG_REDO_BITS;
+      break;
+    case REDO_PTR:
+      context.context = (word)fr->clause;
+      break;
+    case YIELD_PTR:
+      context.context = (word)fr->clause & ~FRG_REDO_MASK;
+      break;
+  }
   context.control = FRG_CUTTED;
   context.engine  = LD;
 
@@ -837,11 +847,7 @@ callCleanupHandler(DECL_LD LocalFrame fr, enum finished reason)
       fr = (LocalFrame)valTermRef(fref);
       clean = consTermRef(argFrameP(fr, 3));
       if ( saveWakeup(&wstate, FALSE) )
-      { static predicate_t PRED_call1 = NULL;
-	int rval;
-
-	if ( !PRED_call1 )
-	  PRED_call1 = PL_predicate("call", 1, "system");
+      { int rval;
 
 	startCritical;
         rval = call_term(contextModule(fr), clean);
@@ -2712,26 +2718,30 @@ PL_open_query(Module ctx, int flags, Procedure proc, term_t args)
   environment_frame = fr;
   qf->parent = LD->query;
   LD->query = qf;
+  qf->qid = allocHeapOrHalt(sizeof(*qf->qid));
+  qf->qid->engine = LD;
+  qf->qid->offset = consTermRef(qf);
 
-  DEBUG(2, Sdprintf("QID=%d\n", QidFromQuery(qf)));
+  DEBUG(2, Sdprintf("QID=%p\n", QidFromQuery(qf)));
   updateAlerted(LD);
 
   return QidFromQuery(qf);
 }
 
 
-#define discard_query(qid) LDFUNC(discard_query, qid)
 static void
-discard_query(DECL_LD qid_t qid)
+discard_query(qid_t qid)
 { QueryFrame qf = QueryFromQid(qid);
 
-  discardChoicesAfter(&qf->frame, FINISH_CUT);
-  qf = QueryFromQid(qid);		/* may be shifted */
-  discardFrame(&qf->frame);
-  if ( true(&qf->frame, FR_WATCHED) )
-  { lTop = (LocalFrame)argFrameP(&qf->frame,
-				 qf->frame.predicate->functor->arity);
-    frameFinished(&qf->frame, FINISH_CUT);
+  WITH_LD(qid->engine)
+  { discardChoicesAfter(&qf->frame, FINISH_CUT);
+    qf = QueryFromQid(qid);		/* may be shifted */
+    discardFrame(&qf->frame);
+    if ( true(&qf->frame, FR_WATCHED) )
+    { lTop = (LocalFrame)argFrameP(&qf->frame,
+				   qf->frame.predicate->functor->arity);
+      frameFinished(&qf->frame, FINISH_CUT);
+    }
   }
 }
 
@@ -2771,26 +2781,32 @@ restore_after_query(QueryFrame qf)
 
 int
 PL_cut_query(qid_t qid)
-{ GET_LD
-  QueryFrame qf = QueryFromQid(qid);
-  int rc = TRUE;
+{ int rc = TRUE;
 
-  DEBUG(CHK_SECURE, assert(qf->magic == QID_MAGIC));
-  if ( qf->foreign_frame )
-    PL_close_foreign_frame(qf->foreign_frame);
+  if ( qid )
+  { WITH_LD(qid->engine)
+    { QueryFrame qf = QueryFromQid(qid);
 
-  if ( false(qf, PL_Q_DETERMINISTIC) )
-  { int exbefore = (exception_term != 0);
+      DEBUG(0, assert(qf->magic == QID_MAGIC));
+      if ( qf->foreign_frame )
+	PL_close_foreign_frame(qf->foreign_frame);
 
-    discard_query(qid);
-    qf = QueryFromQid(qid);
-    if ( !exbefore && exception_term != 0 )
-      rc = FALSE;
+      if ( false(qf, PL_Q_DETERMINISTIC) )
+      { int exbefore = (exception_term != 0);
+
+	discard_query(qid);
+	qf = QueryFromQid(qid);
+	if ( !exbefore && exception_term != 0 )
+	  rc = FALSE;
+      }
+
+      restore_after_query(qf);
+      qf->magic = QID_CMAGIC;		/* disqualify the frame */
+
+      freeHeap(qid, sizeof(*qid));
+    }
   }
 
-  restore_after_query(qf);
-  qf->magic = 0;			/* disqualify the frame */
-
   return rc;
 }
 
@@ -2799,28 +2815,30 @@ int
 PL_close_query(qid_t qid)
 { int rc = TRUE;
 
-  if ( qid != 0 )
-  { GET_LD
-    QueryFrame qf = QueryFromQid(qid);
+  if ( qid )
+  { WITH_LD(qid->engine)
+    { QueryFrame qf = QueryFromQid(qid);
 
-    DEBUG(CHK_SECURE, assert(qf->magic == QID_MAGIC));
-    if ( qf->foreign_frame )
-      PL_close_foreign_frame(qf->foreign_frame);
+      DEBUG(0, assert(qf->magic == QID_MAGIC));
+      if ( qf->foreign_frame )
+	PL_close_foreign_frame(qf->foreign_frame);
 
-    if ( false(qf, PL_Q_DETERMINISTIC) )
-    { int exbefore = (exception_term != 0);
+      if ( false(qf, PL_Q_DETERMINISTIC) )
+      { int exbefore = (exception_term != 0);
 
-      discard_query(qid);
-      qf = QueryFromQid(qid);
-      if ( !exbefore && exception_term != 0 )
-	rc = FALSE;
+	discard_query(qid);
+	qf = QueryFromQid(qid);
+	if ( !exbefore && exception_term != 0 )
+	  rc = FALSE;
+      }
+
+      if ( !(qf->exception && true(qf, PL_Q_PASS_EXCEPTION)) )
+	Undo(qf->choice.mark);
+
+      restore_after_query(qf);
+      qf->magic = QID_CMAGIC;		/* disqualify the frame */
+      freeHeap(qid, sizeof(*qid));
     }
-
-    if ( !(qf->exception && true(qf, PL_Q_PASS_EXCEPTION)) )
-      Undo(qf->choice.mark);
-
-    restore_after_query(qf);
-    qf->magic = 0;			/* disqualify the frame */
   }
 
   return rc;
@@ -2832,13 +2850,28 @@ PL_current_query(void)
 { GET_LD
 
   if ( HAS_LD )
-  { if ( LD->query )
+  { if ( LD->query && LD->query->magic == QID_MAGIC )
       return QidFromQuery(LD->query);
   }
 
   return 0;
 }
 
+int
+PL_can_yield(void)
+{ GET_LD
+
+  return ( HAS_LD &&
+	   LD->query &&
+	   LD->query->magic == QID_MAGIC &&
+	   true(LD->query, PL_Q_ALLOW_YIELD) );
+}
+
+PL_engine_t
+PL_query_engine(qid_t qid)
+{ return qid->engine;
+}
+
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 PL_exception(qid) is used to extract exceptions   from an query executed
@@ -2874,8 +2907,7 @@ PL_exception(qid_t qid)
 
 term_t
 PL_yielded(qid_t qid)
-{ GET_LD
-  QueryFrame qf = QueryFromQid(qid);
+{ QueryFrame qf = QueryFromQid(qid);
 
   return qf->yield.term;
 }
@@ -2965,6 +2997,10 @@ typedef struct register_file
 # define     __PL_ar_ctx (REGISTERS.pl_ar_ctx)
   struct foreign_context fndet_context;	/* foreign function non-deterministic context */
 # define     FNDET_CONTEXT (REGISTERS.fndet_context)
+#if VMCODE_IS_ADDRESS
+  int	     nop1;			/* See SEPARATE_VMI1 */
+  int	     nop2;			/* See SEPARATE_VMI2 */
+#endif
 #ifdef O_DEBUG
   int	     throwed_from_line;		/* Debugging: line we came from */
 # define     THROWED_FROM_LINE	(REGISTERS.throwed_from_line)
@@ -3012,7 +3048,8 @@ typedef struct register_file
 #define VMH_GOTO(...)		do { _VMH_GOTO(__VA_ARGS__); } while(0)
 #define SOLUTION_RETURN(val)	do { VMI_EXIT; _SOLUTION_RETURN(val); } while(0)
 #define VMI_GOTO_CODE(c)	do { VMI_EXIT; _VMI_GOTO_CODE(c); } while(0)
-#define SEPARATE_VMI		(void)0 /* only needed for !O_VMI_FUNCTIONS && VMCODE_IS_ADDRESS */
+#define SEPARATE_VMI1		(void)0
+#define SEPARATE_VMI2		(void)0
 
 /* By default, instruction and helper prologue/epilogue are empty */
 #define _VMI_PROLOGUE(Name,f,na,a)	;
@@ -3118,13 +3155,18 @@ static vmi_instr jmp_table[] =
 #define _NEXT_INSTRUCTION		DbgPrintInstruction(FR, PC); _VMI_GOTO_CODE(*PC++)
 #define _VMI_GOTO(n)			goto n ## _LBL
 #define _VMI_GOTO_CODE(c)		goto *(void *)(c)
-#undef SEPARATE_VMI
+#undef SEPARATE_VMI1
+#undef SEPARATE_VMI2
 /* This macro must ensure that two identical VMI instructions do not get
  * merged onto the same address by the compiler, causing decompilation
  * which translates the addresses back into the VMI number to fail.
- * initWamTable() verfies this does not happen.
+ * initWamTable() verfies this does not happen.  We have two versions
+ * to deal with jumps from to VMI to the same implementation.
  */
-#define SEPARATE_VMI { static volatile int nop = 0; (void)nop; }
+#define SEPARATE_VMI1 \
+	{ if ( ++REGISTERS.nop1 == 0 ) separate_vmi(REGISTERS.nop1); }
+#define SEPARATE_VMI2 \
+	{ if ( ++REGISTERS.nop2 == 0 ) separate_vmi(REGISTERS.nop2); }
 
 #else /* VMCODE_IS_ADDRESS */
 
@@ -3148,9 +3190,14 @@ API_STUB(int)
 
 int
 PL_next_solution(DECL_LD qid_t qid)
-{ register_file REGISTERS = {.qid = qid};	/* Active registers */
-  Code PC;					/* program counter */
-  exception_frame THROW_ENV;			/* PL_thow() environment */
+{ register_file REGISTERS =
+  { .qid = qid
+#if VMCODE_IS_ADDRESS
+    , .nop1 = 0, .nop2 = 0
+#endif
+  };					/* Active registers */
+  Code PC;				/* program counter */
+  exception_frame THROW_ENV;		/* PL_thow() environment */
 
 #if O_VMI_FUNCTIONS
   register_file *registers = &REGISTERS;
@@ -3205,6 +3252,8 @@ depart_continue() to do the normal thing or to the backtrack point.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
   QF  = QueryFromQid(qid);
+  if ( QF->magic == QID_CMAGIC )
+    return FALSE;
   DEBUG(CHK_SECURE, assert(QF->magic == QID_MAGIC));
   if ( true(QF, PL_Q_DETERMINISTIC) )	/* last one succeeded */
   { fid_t fid = QF->foreign_frame;
@@ -3279,6 +3328,10 @@ variables used in the B_THROW instruction.
       DEBUG(CHK_SECURE, checkStacks(NULL));
       if ( exception_term )
 	THROW_EXCEPTION;
+      if ( QF->yield.term == -1 )		/* foreign yield */
+      { DEF = FR->predicate;
+	QF->yield.term = 0;
+      }
       NEXT_INSTRUCTION;
     } else
       BODY_FAILED;
