@@ -241,6 +241,16 @@ wr_is_symbol(int c, write_options *options)
 	    (options->flags & PL_WRT_BACKQUOTE_IS_SYMBOL)) );
 }
 
+static int
+code_requires_quoted(int c, IOSTREAM *fd, int flags)
+{ if ( c > 0x7f && (flags&PL_WRT_QUOTE_NON_ASCII) )
+    return TRUE;
+  if ( fd && Scanrepresent(c, fd) != 0 )
+    return TRUE;
+
+  return FALSE;
+}
+
 static int
 atomType(atom_t a, write_options *options)
 { Atom atom = atomValue(a);
@@ -248,6 +258,7 @@ atomType(atom_t a, write_options *options)
   size_t len = atom->length;
   IOSTREAM *fd = options ? options->out : NULL;
   Module m = options ? options->module : MODULE_user;
+  int flags = options ? options->flags : 0;
 
   if ( len == 0 )
     return AT_QUOTE;
@@ -255,7 +266,7 @@ atomType(atom_t a, write_options *options)
   if ( isLower(*s) || (true(m, M_VARPREFIX) && isAlpha(*s)) )
   { do
     { for( ++s;
-	   --len > 0 && isAlpha(*s) && (!fd || Scanrepresent(*s, fd)==0);
+	   --len > 0 && isAlpha(*s) && !code_requires_quoted(*s, fd, flags);
 	   s++)
 	;
     } while ( len >= 2 &&
@@ -277,7 +288,7 @@ atomType(atom_t a, write_options *options)
 
     for( ;
 	 left > 0 && wr_is_symbol(*s, options) &&
-	 (!fd || Scanrepresent(*s, fd)==0);
+	 !code_requires_quoted(*s, fd, flags);
 	 s++, left--)
       ;
     if ( left > 0 )
@@ -311,17 +322,20 @@ unquoted_atomW(atom_t atom, IOSTREAM *fd, int flags)
   if ( !f_is_prolog_atom_start(*s) )
   { for( ; len > 0; s++, len--)
     { if ( !f_is_prolog_symbol(*s) ||
-	   (fd && Scanrepresent(*s, fd)<0) )
+	   code_requires_quoted(*s, fd, flags) )
 	return FALSE;
     }
     return TRUE;
   }
 
+  if ( code_requires_quoted(*s, fd, flags) )
+    return FALSE;
+
   do
   { for( ++s;
 	 ( --len > 0 &&
 	   f_is_prolog_identifier_continue(*s) &&
-	   (!fd || Scanrepresent(*s, fd)==0)
+	   !code_requires_quoted(*s, fd, flags)
 	 );
 	 s++)
       ;
@@ -523,7 +537,8 @@ PutCloseBrace(IOSTREAM *s)
 static bool
 putQuoted(int c, int quote, int flags, IOSTREAM *stream)
 { if ( (flags & PL_WRT_CHARESCAPES) )
-  { if ( !unicode_quoted_escape(c) && c != quote && c != '\\' )
+  { if ( !unicode_quoted_escape(c) && c != quote && c != '\\' &&
+	 (!stream || Scanrepresent(c, stream) == 0) )
     { TRY(Putc(c, stream));
     } else
     { char esc[10];			/* Longest is UXXXXXXXX */
@@ -1874,6 +1889,7 @@ writeBlobMask(atom_t a)
 
 static const opt_spec write_term_options[] =
 { { ATOM_quoted,		    OPT_BOOL },
+  { ATOM_quote_non_ascii,	    OPT_BOOL },
   { ATOM_ignore_ops,		    OPT_BOOL },
   { ATOM_dotlists,		    OPT_BOOL },
   { ATOM_brace_terms,		    OPT_BOOL },
@@ -1905,13 +1921,14 @@ pl_write_term3(term_t stream, term_t term, term_t opts)
   bool quoted     = FALSE;
   bool ignore_ops = FALSE;
   bool dotlists   = FALSE;
-  bool braceterms = FALSE;
+  bool braceterms = TRUE;
   bool numbervars = -1;			/* not set */
   bool portray    = FALSE;
   term_t gportray = 0;
   atom_t bq       = 0;
   bool charescape = -1;			/* not set */
   bool charescape_unicode = -1;
+  bool quote_non_ascii = FALSE;
   atom_t mname    = ATOM_user;
   atom_t attr     = ATOM_nil;
   atom_t blobs    = ATOM_nil;
@@ -1931,7 +1948,7 @@ pl_write_term3(term_t stream, term_t term, term_t opts)
   options.spacing = ATOM_standard;
 
   if ( !scan_options(opts, 0, ATOM_write_option, write_term_options,
-		     &quoted, &ignore_ops, &dotlists, &braceterms,
+		     &quoted, &quote_non_ascii, &ignore_ops, &dotlists, &braceterms,
 		     &numbervars, &portray, &portray, &gportray,
 		     &charescape, &charescape_unicode,
 		     &options.max_depth, &mname,
@@ -1998,14 +2015,15 @@ pl_write_term3(term_t stream, term_t term, term_t opts)
   if ( numbervars == -1 )
     numbervars = (portray ? TRUE : FALSE);
 
-  if ( quoted )     options.flags |= PL_WRT_QUOTED;
-  if ( ignore_ops ) options.flags |= PL_WRT_IGNOREOPS;
-  if ( dotlists )   options.flags |= PL_WRT_DOTLISTS;
-  if ( braceterms ) options.flags |= PL_WRT_BRACETERMS;
-  if ( numbervars ) options.flags |= PL_WRT_NUMBERVARS;
-  if ( portray )    options.flags |= PL_WRT_PORTRAY;
-  if ( !cycles )    options.flags |= PL_WRT_NO_CYCLES;
-  if ( no_lists )   options.flags |= PL_WRT_NO_LISTS;
+  if ( quoted )          options.flags |= PL_WRT_QUOTED;
+  if ( quote_non_ascii ) options.flags |= PL_WRT_QUOTE_NON_ASCII;
+  if ( ignore_ops )      options.flags |= PL_WRT_IGNOREOPS;
+  if ( dotlists )        options.flags |= PL_WRT_DOTLISTS;
+  if ( !braceterms )     options.flags |= PL_WRT_BRACETERMS;
+  if ( numbervars )      options.flags |= PL_WRT_NUMBERVARS;
+  if ( portray )         options.flags |= PL_WRT_PORTRAY;
+  if ( !cycles )         options.flags |= PL_WRT_NO_CYCLES;
+  if ( no_lists )        options.flags |= PL_WRT_NO_LISTS;
   if ( bq )
   { unsigned int flags = 0;
 
@@ -2163,8 +2181,9 @@ pl_write_canonical2(term_t stream, term_t term)
 
   rc = ( numberVars(term, &options, 0) != NV_ERROR &&
 	 do_write2(stream, term,
-		   PL_WRT_QUOTED|PL_WRT_IGNOREOPS|PL_WRT_NUMBERVARS|
-		   PL_WRT_NODOTINATOM, TRUE)
+		   PL_WRT_QUOTED|PL_WRT_QUOTE_NON_ASCII|
+		   PL_WRT_IGNOREOPS|PL_WRT_NUMBERVARS|
+		   PL_WRT_NODOTINATOM|PL_WRT_BRACETERMS, TRUE)
        );
 
   END_NUMBERVARS(TRUE);
